******* Borenstein, AEJ-EP, "The Redistributional Impact of Non-Linear Electricity Pricing"
******* See BorensteinAEJEPReadMe.txt for description
******* PROGRAM #6 BILLCALC5.DO

clear
capture log close
set more off
set mem 1200m
log using "billcalc5.slf", replace text

u SCEannual
drop rnd

*replace CARE=0

summ days flaguse
drop if flaguse~=1
drop flaguse
summ days

g used=use/days

**SET BASIC PROPENSITIES TO SIGN UP FOR CARE NOW BASED ON 78% PENETRATION
scalar CAREshr1=0.7359
scalar CAREshr2=0.4506
scalar CAREshr3=0.1008
scalar CAREshr4=0.0063
scalar CAREshr5=0.0001

sort bg
merge bg using c:\billtiers\income_bg_2008_0301.dta
tab _merge
drop if _merge~=3
drop _merge
drop if p_under_20==.

sort bg
egen bgNCARE=sum((1-CARE)*days), by (bg)
egen bgCARE=sum(CARE*days), by (bg)
egen bgdays=sum(days), by (bg)

**SET SCALE FACTOR TO DISTRIBUTE CARE CUSTOMERS PROPORTIONALLY TO PROPENSITIES
g CAREscale=bgCARE / (bgdays * (CAREshr1*p_under_20+CAREshr2*p_20_40+CAREshr3*p_40_60+CAREshr4*p_60_100+CAREshr5*p_over_100))

g CARE1= CAREshr1*CAREscale*bgdays*p_under_20
g CARE2= CAREshr2*CAREscale*bgdays*p_20_40
g CARE3= CAREshr3*CAREscale*bgdays*p_40_60
g CARE4= CAREshr4*CAREscale*bgdays*p_60_100
g CARE5= CAREshr5*CAREscale*bgdays*p_over_100

summ CARE*

**FIX CASES IN WHICH SCALE FACTOR TAKE CARE DAYS ABOVE TOTAL DAYS IN INCOME BRACKET
**BY DISTRIBUTING DAYS TO NEXT HIGHEST INCOME BRACKET

replace CARE1=bgdays*p_under_20 if CARE1>bgdays*p_under_20
replace CARE2=bgdays*p_20_40 if CARE2>bgdays*p_20_40
replace CARE3=bgdays*p_40_60 if CARE3>bgdays*p_40_60
replace CARE4=bgdays*p_60_100 if CARE4>bgdays*p_60_100
replace CARE5=bgdays*p_over_100 if CARE5>bgdays*p_over_100

g totC=CARE1+CARE2+CARE3+CARE4+CARE5
replace CARE1=CARE1+(bgCARE-totC) if bgCARE>totC
replace CARE1=bgdays*p_under_20 if CARE1>bgdays*p_under_20

replace totC=CARE1+CARE2+CARE3+CARE4+CARE5
replace CARE2=CARE2+(bgCARE-totC) if bgCARE>totC
replace CARE2=bgdays*p_20_40 if CARE2>bgdays*p_20_40

replace totC=CARE1+CARE2+CARE3+CARE4+CARE5
replace CARE3=CARE3+(bgCARE-totC) if bgCARE>totC
replace CARE3=bgdays*p_40_60 if CARE3>bgdays*p_40_60

replace totC=CARE1+CARE2+CARE3+CARE4+CARE5
replace CARE4=CARE4+(bgCARE-totC) if bgCARE>totC
replace CARE4=bgdays*p_60_100 if CARE4>bgdays*p_60_100

replace totC=CARE1+CARE2+CARE3+CARE4+CARE5
replace CARE5=CARE5+(bgCARE-totC) if bgCARE>totC
replace CARE5=bgdays*p_over_100 if CARE5>bgdays*p_over_100

summ CARE*

replace totC=CARE1+CARE2+CARE3+CARE4+CARE5
g xx=bgCARE-totC
summ xx,d
drop xx

g NCARE1=max(0 , bgdays*p_under_20-CARE1)
g NCARE2=max(0 ,bgdays*p_20_40-CARE2)
g NCARE3=max(0 ,bgdays*p_40_60-CARE3)
g NCARE4=max(0 ,bgdays*p_60_100-CARE4)
g NCARE5=max(0 ,bgdays*p_over_100-CARE5)

g tot=CARE1+CARE2+CARE3+CARE4+CARE5+NCARE1+NCARE2+NCARE3+NCARE4+NCARE5
g totN=NCARE1+NCARE2+NCARE3+NCARE4+NCARE5
summ tot bgdays totC bgCARE totN bgNCARE if tot~=. & bgdays~=.

drop tot totC totN

sort bg used
g rankU=1 if _n==1
replace rankU=1 if bg~=bg[_n-1]
replace rankU=rankU[_n-1]+1 if rankU==.

set seed 2126423689
g rnd=uniform()
sort bg rnd
g rankR=1 if _n==1
replace rankR=1 if bg~=bg[_n-1]
replace rankR=rankR[_n-1]+1 if rankR==.

save yyy, replace


***  ASSIGNING INCOME BY WEIGHTED RANK METHOD


local wgt=-30
while `wgt'<=100{

*local wgt=0

u yyy

g w=float(`wgt')/100

summ w

g rankW=(1-w)*rankR+w*rankU
sort bg rankW

summ rank*

g bgdaysN=.
g bgdaysC=.

replace bgdaysN=(1-CARE)*days if _n==1
replace bgdaysN=(1-CARE)*days if bg~=bg[_n-1]
replace bgdaysC=(CARE)*days if _n==1
replace bgdaysC=(CARE)*days if bg~=bg[_n-1]

replace bgdaysN=bgdaysN[_n-1]+(1-CARE)*days if bgdaysN==.
replace bgdaysC=bgdaysC[_n-1]+(CARE)*days if bgdaysC==.

g incCAREw=.
g incNCAREw=.

replace incCAREw=1 if 0.5*(bgdaysC+bgdaysC[_n-1])<CARE1 & CARE==1
replace incCAREw=2 if 0.5*(bgdaysC+bgdaysC[_n-1])>=CARE1 & 0.5*(bgdaysC+bgdaysC[_n-1])<CARE1+CARE2 & CARE==1
replace incCAREw=3 if 0.5*(bgdaysC+bgdaysC[_n-1])>=CARE1+CARE2 & 0.5*(bgdaysC+bgdaysC[_n-1])<CARE1+CARE2+CARE3 & CARE==1
replace incCAREw=4 if 0.5*(bgdaysC+bgdaysC[_n-1])>=CARE1+CARE2+CARE3 & 0.5*(bgdaysC+bgdaysC[_n-1])<CARE1+CARE2+CARE3+CARE4 & CARE==1
replace incCAREw=5 if 0.5*(bgdaysC+bgdaysC[_n-1])>=CARE1+CARE2+CARE3+CARE4 & CARE==1

replace incCAREw=1 if 0.5*(bgdaysC)<CARE1 & CARE==1 & bg~=bg[_n-1]
replace incCAREw=2 if 0.5*(bgdaysC)>=CARE1 & 0.5*(bgdaysC)<CARE1+CARE2 & CARE==1 & bg~=bg[_n-1]
replace incCAREw=3 if 0.5*(bgdaysC)>=CARE1+CARE2 & 0.5*(bgdaysC)<CARE1+CARE2+CARE3 & CARE==1 & bg~=bg[_n-1]
replace incCAREw=4 if 0.5*(bgdaysC)>=CARE1+CARE2+CARE3 & 0.5*(bgdaysC)<CARE1+CARE2+CARE3+CARE4 & CARE==1 & bg~=bg[_n-1]
replace incCAREw=5 if 0.5*(bgdaysC)>=CARE1+CARE2+CARE3+CARE4 & CARE==1 & bg~=bg[_n-1]

summ inc* CARE*

replace incNCAREw=1 if 0.5*(bgdaysN+bgdaysN[_n-1])<NCARE1 & CARE==0
replace incNCAREw=2 if 0.5*(bgdaysN+bgdaysN[_n-1])>=NCARE1 & 0.5*(bgdaysN+bgdaysN[_n-1])<NCARE1+NCARE2 & CARE==0
replace incNCAREw=3 if 0.5*(bgdaysN+bgdaysN[_n-1])>=NCARE1+NCARE2 & 0.5*(bgdaysN+bgdaysN[_n-1])<NCARE1+NCARE2+NCARE3 & CARE==0
replace incNCAREw=4 if 0.5*(bgdaysN+bgdaysN[_n-1])>=NCARE1+NCARE2+NCARE3 & 0.5*(bgdaysN+bgdaysN[_n-1])<NCARE1+NCARE2+NCARE3+NCARE4 & CARE==0
replace incNCAREw=5 if 0.5*(bgdaysN+bgdaysN[_n-1])>=NCARE1+NCARE2+NCARE3+NCARE4 & CARE==0

replace incNCAREw=1 if 0.5*(bgdaysN)<NCARE1 & CARE==0 & bg~=bg[_n-1]
replace incNCAREw=2 if 0.5*(bgdaysN)>=NCARE1 & 0.5*(bgdaysN)<NCARE1+NCARE2 & CARE==0 & bg~=bg[_n-1]
replace incNCAREw=3 if 0.5*(bgdaysN)>=NCARE1+NCARE2 & 0.5*(bgdaysN)<NCARE1+NCARE2+NCARE3 & CARE==0 & bg~=bg[_n-1]
replace incNCAREw=4 if 0.5*(bgdaysN)>=NCARE1+NCARE2+NCARE3 & 0.5*(bgdaysN)<NCARE1+NCARE2+NCARE3+NCARE4 & CARE==0 & bg~=bg[_n-1]
replace incNCAREw=5 if 0.5*(bgdaysN)>=NCARE1+NCARE2+NCARE3+NCARE4 & CARE==0 & bg~=bg[_n-1]

summ inc* NCARE*

tab incCAREw [aw=days]
tab incNCAREw [aw=days]

drop CARE1 CARE2 CARE3 CARE4 CARE5 NCARE1 NCARE2 NCARE3 NCARE4 NCARE5 rnd  bgdaysN bgdaysC p_* bgCARE bgNCARE bgdays

g incW=incCAREw if CARE==1
replace incW=incNCAREw if CARE==0

drop incCARE* incNCARE*

drop used
sort CARE incW region
collapse (sum) days use* (mean) w, by(CARE incW region)
save zzz, replace

summ w
g used=use/days
sort region incW used w CARE
list region incW used w CARE
keep region incW days use used w CARE
sort region incW CARE
capture append using rerank5
save rerank5, replace
clear


** DATA FOR WEIGHTED RANK

u altrates 
keep CARE rate*
save xxx, replace

u zzz
sort CARE incW
collapse (sum) days use* (mean) w, by(CARE incW)
g used=use/days
sort CARE
merge CARE using xxx

g bill5=rate51*use1+rate52*use2+rate53*use3+rate54*use4+rate55*use5
g bill2=rate21*use1+rate22*(use2+use3+use4+use5)
g bill1=rate11*(use1+use2+use3+use4+use5)

g annbill5=365*(bill5/days)
g annbill2=365*(bill2/days)
g annbill1=365*(bill1/days)

egen totrev5=sum(bill5), by(CARE)
egen totrev2=sum(bill2), by(CARE)
egen totrev1=sum(bill1), by(CARE)

g revshr5=bill5/totrev5
g revshr2=bill2/totrev2
g revshr1=bill1/totrev1

g avgP5=bill5/use
g avgP2=bill2/use
g avgP1=bill1/use

sort CARE incW

list CARE incW annbill*

list CARE incW revshr*

list CARE incW avgP*

egen totdays=sum(days)
g daysshr=days/totdays
list CARE incW used daysshr


keep w CARE incW days use bill* annb* avgP*
save www, replace

capture append using billcalc5
save billcalc5, replace


***now combining CARE and non-CARE of the same income bracket
u www
g Cdays=CARE*days
sort incW
collapse (sum) Cdays days use* bill* (mean) w, by(incW)
g used=use/days

g annbill5=365*(bill5/days)
g annbill2=365*(bill2/days)
g annbill1=365*(bill1/days)

egen totrev5=sum(bill5)
egen totrev2=sum(bill2)
egen totrev1=sum(bill1)

g revshr5=bill5/totrev5
g revshr2=bill2/totrev2
g revshr1=bill1/totrev1

g avgP5=bill5/use
g avgP2=bill2/use
g avgP1=bill1/use

list incW annbill*

list incW revshr*

list incW avgP*

egen totdays=sum(days)
g daysshr=days/totdays
g Cshr=Cdays/days
list incW used daysshr Cdays

g CARE=2

keep w CARE incW days use bill* annb* avgP*
capture append using billcalc5
save billcalc5, replace

clear


** DATA FOR WEIGHTED RANK -- CALCULATED FOR NO CARE PROGRAM (BUT USING SAME BRACKET ASSIGNMENTS AS W CARE)

u altratesa
keep CARE rate*
save xxx, replace

u zzz
sort CARE incW
collapse (sum) days use* (mean) w, by(incW)
g used=use/days
g CARE=0
sort CARE
merge CARE using xxx

g bill5=rate51A*use1+rate52A*use2+rate53A*use3+rate54A*use4+rate55A*use5
g bill2=rate21A*use1+rate22A*(use2+use3+use4+use5)
g bill1=rate11A*(use1+use2+use3+use4+use5)

g annbill5=365*(bill5/days)
g annbill2=365*(bill2/days)
g annbill1=365*(bill1/days)

egen totrev5=sum(bill5), by(CARE)
egen totrev2=sum(bill2), by(CARE)
egen totrev1=sum(bill1), by(CARE)

summ totrev* if CARE==0

g revshr5=bill5/totrev5
g revshr2=bill2/totrev2
g revshr1=bill1/totrev1

g avgP5=bill5/use
g avgP2=bill2/use
g avgP1=bill1/use

sort CARE incW

list CARE incW annbill*

list CARE incW revshr*

list CARE incW avgP*

egen totdays=sum(days), by(CARE)
g daysshr=days/totdays
list CARE incW used daysshr


replace CARE=-1

keep w CARE incW days use bill* annb* avgP*
capture append using billcalc5
save billcalc5, replace

clear


local wgt=`wgt'+1

}

exit


